import java.util.Random;

DCEL dcel = new DCEL();

int to_x(double coordX) {
  return (int)Math.round(coordX + (width / 2));
}

int to_y(double coordY) {
  return (int)Math.round((height/2) - coordY);
}

void setup() {
  size(800, 600);
  smooth(4);
  background(255);

  // Create your DCEL here:
  
  // Create the six vertices A--F: 
  DCEL.Vertex A = dcel.new Vertex(new Point2d(-300, -150));
  DCEL.Vertex B = dcel.new Vertex(new Point2d(0, -150));
  DCEL.Vertex C = dcel.new Vertex(new Point2d(300, -150));
  DCEL.Vertex D = dcel.new Vertex(new Point2d(300, 150));
  DCEL.Vertex E = dcel.new Vertex(new Point2d(0, 150));
  DCEL.Vertex F = dcel.new Vertex(new Point2d(-300, 150));
  
  // Create a face for the inner face; Note: the outer face is dcel.outerFace
  DCEL.Face f = dcel.new Face();
  
  // Create the six half-edges for inner face
  DCEL.HalfEdge AB = dcel.new HalfEdge(A, f);
  DCEL.HalfEdge BC = dcel.new HalfEdge(B, f);
  DCEL.HalfEdge CD = dcel.new HalfEdge(C, f);
  DCEL.HalfEdge DE = dcel.new HalfEdge(D, f);
  DCEL.HalfEdge EF = dcel.new HalfEdge(E, f);
  DCEL.HalfEdge FA = dcel.new HalfEdge(F, f);
  
  // Make sure you set an incident half edge for f: 
  f.incidentEdge = AB;
  
  // Set up the next/prev pointers correctly for the face: 
  AB.next = BC; BC.prev = AB;
  BC.next = CD; CD.prev = BC;
  CD.next = DE; DE.prev = CD;
  DE.next = EF; EF.prev = DE;
  EF.next = FA; FA.prev = EF;
  FA.next = AB; AB.prev = FA;
  
  // Create the six half-edges for the outer face
  DCEL.HalfEdge AF = dcel.new HalfEdge(A, dcel.outerFace);
  DCEL.HalfEdge FE = dcel.new HalfEdge(F, dcel.outerFace);
  DCEL.HalfEdge ED = dcel.new HalfEdge(E, dcel.outerFace);
  DCEL.HalfEdge DC = dcel.new HalfEdge(D, dcel.outerFace);
  DCEL.HalfEdge CB = dcel.new HalfEdge(C, dcel.outerFace);
  DCEL.HalfEdge BA = dcel.new HalfEdge(B, dcel.outerFace);
  
  // Make sure you set an incident half-edge for the outer face: 
  dcel.outerFace.incidentEdge = AF;
  
  // Set up the next/prev pointers correctly for the face: 
  AF.next = FE; FE.prev = AF;
  FE.next = ED; ED.prev = FE;
  ED.next = DC; DC.prev = ED;
  DC.next = CB; CB.prev = DC;
  CB.next = BA; BA.prev = CB;
  BA.next = AF; AF.prev = BA;
  
  // Set twins between the six inner and six outer half-edges
  AB.twin = BA; BA.twin = AB;
  BC.twin = CB; CB.twin = BC;
  CD.twin = DC; DC.twin = CD;
  DE.twin = ED; ED.twin = DE;
  EF.twin = FE; FE.twin = EF;
  FA.twin = AF; AF.twin = FA;
  
  // Test your split face code: 
  // TODO
}

void draw() {
  background(255);
  Random rand = new Random(12345679);
  // Draw faces
  boolean blueFace = false;
  for (DCEL.Face f : dcel.faces) {
    if (f != dcel.outerFace) {
      fill(rand.nextInt(256), rand.nextInt(256), rand.nextInt(256));
      ArrayList<DCEL.HalfEdge> edges = dcel.getCycleFrom(f.incidentEdge);
      if (edges != null) {
        beginShape();
        for (DCEL.HalfEdge he : edges) {
          vertex(to_x(he.origin.coord.x), to_y(he.origin.coord.y));
        }
        endShape(CLOSE);
      }
    }
  }
  
  // Draw edges
  stroke(0);
  for (DCEL.HalfEdge e : dcel.halfEdges) {
    line(to_x(e.origin.coord.x), to_y(e.origin.coord.y), 
            to_x(e.twin.origin.coord.x), to_y(e.twin.origin.coord.y));
  }
  
  // Draw vertices
  fill(255);
  stroke(0);
  for (DCEL.Vertex v : dcel.vertices) {
    ellipse(to_x(v.coord.x), to_y(v.coord.y), 8, 8);
  }
}